import { TSDeclareFunction, TSInterfaceDeclaration, TSTypeAliasDeclaration, TSEnumDeclaration, TSModuleDeclaration, Statement, TSCallSignatureDeclaration, TSConstructSignatureDeclaration, TSMethodSignature, TSType, TSPropertySignature, TSInterfaceBody, TSTypeLiteral, TSIntersectionType, TSTypeElement, TSParenthesizedType, Identifier } from '@babel/types';

declare type TSDeclaration = TSDeclareFunction | TSInterfaceDeclaration | TSTypeAliasDeclaration | TSEnumDeclaration | TSModuleDeclaration;
interface TSFile {
    filePath: string;
    content: string;
    ast: Statement[];
}
interface TSProperties {
    callSignatures: Array<{
        type: TSCallSignatureDeclaration;
        file: TSFile;
    }>;
    constructSignatures: Array<{
        type: TSConstructSignatureDeclaration;
        file: TSFile;
    }>;
    methods: Record<string | number, Array<{
        type: TSMethodSignature;
        file: TSFile;
    }>>;
    properties: Record<string | number, {
        value: {
            type: TSType;
            file: TSFile;
        } | null;
        optional: boolean;
        signature: {
            type: TSPropertySignature;
            file: TSFile;
        };
    }>;
}
declare const tsFileCache: Record<string, TSFile>;
declare function getTSFile(filePath: string): Promise<TSFile>;
declare const isTSDeclaration: (node: any) => node is TSDeclaration;
declare function mergeTSProperties(a: TSProperties, b: TSProperties): TSProperties;
/**
 * get properties of `interface` or `type` declaration
 *
 * @limitation don't support index signature
 */
declare function resolveTSProperties({ type, file, }: TSResolvedType<TSInterfaceDeclaration | TSInterfaceBody | TSTypeLiteral | TSIntersectionType>): Promise<TSProperties>;
/**
 * @limitation don't support index signature
 */
declare function resolveTypeElements(file: TSFile, elements: Array<TSTypeElement>): TSProperties;
interface TSResolvedType<T = Exclude<TSType, TSParenthesizedType> | Exclude<TSDeclaration, TSTypeAliasDeclaration>> {
    file: TSFile;
    type: T;
}
/**
 * Resolve a reference to a type.
 *
 * Supports `type` and `interface` only.
 *
 * @limitation don't support non-TS declaration (e.g. class, function...)
 */
declare function resolveTSReferencedType({ file, type, }: TSResolvedType<TSType | Identifier | TSDeclaration>): Promise<TSResolvedType | undefined>;
declare type TSFileExports = Record<string, TSResolvedType | undefined>;
declare const tsFileExportsCache: Map<TSFile, TSFileExports>;
/**
 * Get exports of the TS file.
 *
 * @limitation don't support non-TS declaration (e.g. class, function...)
 * @limitation don't support `export default`, since TS don't support it currently.
 * @limitation don't support `export * as xxx from '...'` (aka namespace).
 */
declare function resolveTSFileExports(file: TSFile): Promise<TSFileExports>;
declare type ResolveTSFileIdImpl = (id: string, importer: string) => Promise<string | undefined> | string | undefined;
declare function resolveTSFileId(id: string, importer: string): string | Promise<string | undefined> | undefined;
/**
 * @limitation don't node_modules and JavaScript file
 */
declare function resolveTSFileIdNode(id: string, importer: string): string | undefined;
declare function setResolveTSFileIdImpl(impl: ResolveTSFileIdImpl): void;

export { ResolveTSFileIdImpl, TSDeclaration, TSFile, TSFileExports, TSProperties, TSResolvedType, getTSFile, isTSDeclaration, mergeTSProperties, resolveTSFileExports, resolveTSFileId, resolveTSFileIdNode, resolveTSProperties, resolveTSReferencedType, resolveTypeElements, setResolveTSFileIdImpl, tsFileCache, tsFileExportsCache };
